import ipkiss3.all as i3
import numpy as np

np.savetxt("n_eff_tabulated_data.csv", np.asarray([[1.1, 1.3, 1.5, 1.7], [2.3, 2.32, 2.34, 2.4]]).T)


class NeffModel(i3.CompactModel):
    parameters = ["length", "n_eff_data"]

    terms = [i3.OpticalTerm(name="in"), i3.OpticalTerm(name="out")]

    def calculate_smatrix(parameters, env, S):
        n_eff_data = parameters.n_eff_data
        # Simple linear interpolation: find index nearest to the wavelength, calculate fraction, interpolate.
        idx = np.argmin(np.abs(n_eff_data[:, 0] - env.wavelength))
        if n_eff_data[idx, 0] > env.wavelength:
            idx = idx - 1
        frac = (env.wavelength - n_eff_data[idx, 0]) / (n_eff_data[idx + 1, 0] - n_eff_data[idx, 0])
        n_eff = frac * n_eff_data[idx, 1] + (1 - frac) * n_eff_data[idx + 1, 1]
        S["in", "out"] = S["out", "in"] = np.exp(1j * 2 * np.pi * parameters.length / env.wavelength * n_eff)


class MyPCell(i3.PCell):
    class Netlist(i3.NetlistView):
        def _generate_netlist(self, nl):
            nl += i3.OpticalTerm(name="in")
            nl += i3.OpticalTerm(name="out")
            return nl

    class CircuitModel(i3.CircuitModelView):
        filename = i3.StringProperty(default="n_eff_tabulated_data.csv")
        length = i3.NonNegativeNumberProperty(default=10)
        n_eff_data = i3.NumpyArrayProperty(doc="Effective index, 2D array of (wavelengths, n_eff)")

        def _default_n_eff_data(self):
            return np.loadtxt(self.filename)

        def _generate_model(self):
            return NeffModel(length=self.length, n_eff_data=self.n_eff_data)


pcell = MyPCell()
cm = pcell.CircuitModel()

S = cm.get_smatrix(wavelengths=[1.4, 1.5, 1.55])

print(S["in", "out"])
